#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: savechanges.pl,v 1.0.0.0 Sat 21 Jun 2008 08:25:02 AM BST mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# Handler which writes / updates the drbd and HA config files.
#

use CCE;
use POSIX qw(isalpha);

my $DEBUG = 0;

my $cce = new CCE('Domain' => 'vz-vserver');

if ($DEBUG == 0) {
    $cce->connectfd();
}
else {
    $cce->connectuds(); # only for debugging
}

$drbd_conf = '/etc/drbd.conf';
$ha_conf = '/etc/ha.d/ha.cf';

$cluster = $cce->event_object();
$cluster_new = $cce->event_new();
$cluster_old = $cce->event_old();

($sys_oid) = $cce->find('Cluster', '');
($ok, $system) = $cce->get($sys_oid);

# Do the deeds:
&use_gui_input;

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);

# Use GUI submitted new individual values:
sub use_gui_input {

	vps_write_conf(
    	    $system->{'degr_wfc_timeout'},
    	    $system->{'rate'},
    	    $system->{'Std_Pri_fqdn'},
    	    $system->{'Std_Pri_Device'},
    	    $system->{'Std_Pri_Disk'},
    	    $system->{'Std_Pri_ipAddr'},
    	    $system->{'Std_Sec_fqdn'},
    	    $system->{'Std_Sec_Device'},
    	    $system->{'Std_Sec_Disk'},
    	    $system->{'Std_Sec_ipAddr'},

    	    $system->{'keepalive'},
    	    $system->{'warntime'},
   	    $system->{'deadtime'},
    	    $system->{'initdead'},
    	    $system->{'ucast_eth1_master'},
    	    $system->{'ucast_eth1_slave'},
    	    $system->{'ucast_eth0_master'},
    	    $system->{'ucast_eth0_slave'},
    	    $system->{'auto_failback'},
    	    $system->{'ping'}
        );
}

sub vps_write_conf {

        $degr_wfc_timeout   	= shift;
        $rate            	= shift;
        $Std_Pri_fqdn       	= shift;
        $Std_Pri_Device     	= shift;
        $Std_Pri_Disk       	= shift;
        $Std_Pri_ipAddr      = shift;
        $Std_Sec_fqdn        = shift;
        $Std_Sec_Device      = shift;
        $Std_Sec_Disk        = shift;
        $Std_Sec_ipAddr      = shift;

        $keepalive           = shift;
        $warntime            = shift;
        $deadtime            = shift;
        $initdead            = shift;
        $ucast_eth1_master   = shift;
        $ucast_eth1_slave    = shift;
        $ucast_eth0_master   = shift;
        $ucast_eth0_slave    = shift;
        $auto_failback       = shift;
        $ping                = shift;

	@myfqdn = split(/\./, $Std_Pri_fqdn);
	$myhost = $myfqdn[0];

	if ( -d "/usr/share/doc/drbd-0.7.22" ) {
		# Write drbd.conf for 0.7:
		&oldconf;
	}

	if ( -d "/usr/share/doc/drbd82-8.2.6" ) {
		# Write drbd.conf for 0.8:
		&newconf;
	}

	open(CONF, ">$ha_conf") || die "Can't write to $drbd_conf!";
	print CONF "#
# Heartbeat configuration:
#

logfacility daemon

# Heartbeat cluster members
node $Std_Pri_fqdn
node $Std_Sec_fqdn

# Heartbeat communication timing
keepalive $keepalive
warntime $warntime
deadtime $deadtime
initdead $initdead

# Heartbeat communication paths
udpport 694
ucast eth1 $ucast_eth1_master
ucast eth1 $ucast_eth1_slave
ucast eth0 $ucast_eth0_master
ucast eth0 $ucast_eth0_slave
#baud 19200
#serial /dev/ttyS1

# Fail back automatically
auto_failback $auto_failback

# Monitoring of network connection to default gateway
ping $ping
respawn hacluster /usr/lib64/heartbeat/ipfail
";
	close CONF;


    # Restart Daemons:
    system("/etc/init.d/drbd reload");
    system("/etc/init.d/heartbeat reload");
	
}


#
## New subs:
#

# Write drbd.conf for 0.7:

sub oldconf {

        open(CONF, ">$drbd_conf") || die "Can't write to $drbd_conf!";
        print CONF "#
# please have a a look at the example configuration file in
# /usr/share/doc/drbd/drbd.conf
#

resource vz {
  protocol C;
  incon-degr-cmd \"echo 'DRBD: Primary on $myhost degraded!' | wall ; sleep 60 ; halt -f\";

  startup {
    degr-wfc-timeout $degr_wfc_timeout;
  }

  net {
    on-disconnect reconnect;
  }

  disk {
    on-io-error   detach;
  }

  syncer {
    rate $rate;
    group 1;
    al-extents 257;
  }

  on $Std_Pri_fqdn {
    device     $Std_Pri_Device;
    disk       $Std_Pri_Disk;
    address    $Std_Pri_ipAddr:7788;
    meta-disk  internal;
  }

  on $Std_Sec_fqdn {
    device     $Std_Sec_Device;
    disk       $Std_Sec_Disk;
    address    $Std_Sec_ipAddr:7788;
    meta-disk  internal;
  }

}
";
	close CONF;

}

# Write drbd.conf for 0.8:

sub newconf {
        open(CONF, ">$drbd_conf") || die "Can't write to $drbd_conf!";
        print CONF "#
# please have a a look at the example configuration file in
# /usr/share/doc/drbd82-8.2.6/drbd.conf
#

global {
  usage-count no;
}

resource vz {
  protocol C;

  handlers {

    # what should be done in case the node is primary, degraded
    # (=no connection) and has inconsistent data.
    pri-on-incon-degr \"echo o > /proc/sysrq-trigger ; halt -f\";

    # The node is currently primary, but lost the after split brain
    # auto recovery procedure. As as consequence it should go away.
    pri-lost-after-sb \"echo o > /proc/sysrq-trigger ; halt -f\";

    # In case you have set the on-io-error option to 'call-local-io-error',
    # this script will get executed in case of a local IO error. It is
    # expected that this script will case a immediate failover in the
    # cluster.
    local-io-error \"echo o > /proc/sysrq-trigger ; halt -f\";

  }

  startup {
    degr-wfc-timeout $degr_wfc_timeout;
  }

  disk {
    on-io-error   call-local-io-error;
  }

  syncer {
    rate $rate;
    al-extents 257;
  }

  net {
    after-sb-0pri discard-younger-primary;
    after-sb-1pri consensus;
    after-sb-2pri disconnect;
  }

  on $Std_Pri_fqdn {
    device     $Std_Pri_Device;
    disk       $Std_Pri_Disk;
    address    $Std_Pri_ipAddr:7788;
    meta-disk  internal;
  }

  on $Std_Sec_fqdn {
    device     $Std_Sec_Device;
    disk       $Std_Sec_Disk;
    address    $Std_Sec_ipAddr:7788;
    meta-disk  internal;
  }

}
";
	close CONF;
}


# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


