#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: update_cluster_info.pl, v1.1.0.1 Thu 24 Sep 2009 06:46:59 AM CEST mstauber Exp $
# Copyright 2006-2009 Solarspeed Ltd. All rights reserved.

# This constructor / handler polls /etc/drbd.conf and /proc/drbd for information
# about the cluster configuration and its status. That info is then written into
# CCE for GUI usage.

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

$DEBUG = "0";

$cce->connectuds();

# Location of the drbd.conf file:
$cfgfile = '/etc/drbd.conf';

# Location of the ha.cf file:
$hacfgfile = '/etc/ha.d/ha.cf';

# Location of the drbd proc::
$drbdproc = '/proc/drbd';

# We are starting sane:
$bad_info_detected = '0';

if ( -d "/usr/share/doc/drbd-0.7.22" ) {
	# Parse data for DRBD 0.7.XX:
	&main;
}
if ( -d "/usr/share/doc/drbd82-8.2.6" ) {
	# Parse data for DRBD 0.8.2.X:
	&main_new;
}
if ( -d "/usr/share/doc/drbd82-8.3.0" ) {
	# Parse data for DRBD 0.8.3.X:
	&main_new;
}

# Populate CCE:
&feedthemonster;

$cce->bye('SUCCESS');
exit(0);

##
# Subs:
##

# For DRBD 0.7:
sub main {

    open (F, $cfgfile) || die "Could not open $cfgfile: $!";
    while ($line = <F>) {
	chomp($line);
	next if $line =~ /^\s*$/; 		# skip blank lines
	if ($line =~ /degr-wfc-timeout (.*)\;/) {
    	    $degr_wfc_timeout = $1;
    	}
	if ($line =~ /on-disconnect (.*);/) {
    	    $on_disconnect = $1;
    	}
	if ($line =~ /on-io-error (.*);/) {
    	    $on_io_error = $1;
    	}
	if ($line =~ /rate (.*)M;/) {
    	    $rate = $1 . "M";
    	}
	if (($line =~ /on (.*) \{/) && (!$parsed_Std_Sec_fqdn)) {
    	    $parsed_Std_Pri_fqdn = $1;
    	}
	if (($line =~ /on (.*) \{/) && ($parsed_Std_Pri_fqdn)) {
    	    $parsed_Std_Sec_fqdn = $1;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Device)) {
    	    $parsed_Std_Pri_Device = "/dev/" . $2;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Device)) {
    	    $parsed_Std_Sec_Device = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Pri_Disk = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Sec_Disk = "/dev/" . $2;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && (!$parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Pri_ipAddr = $1;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && ($parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Sec_ipAddr = $1;
    	}
    } 
    close(F);

    if ($DEBUG == "1") {
        print "degr_wfc_timeout	: $degr_wfc_timeout \n";
	print "on_disconnect		: $on_disconnect \n";
        print "rate			: $rate \n";
        print "parsed_Std_Pri_fqdn	: $parsed_Std_Pri_fqdn \n";
        print "parsed_Std_Sec_fqdn	: $parsed_Std_Sec_fqdn \n";
        print "parsed_Std_Pri_Device	: $parsed_Std_Pri_Device \n";
        print "parsed_Std_Sec_Device	: $parsed_Std_Sec_Device \n";
        print "parsed_Std_Pri_Disk	: $parsed_Std_Pri_Disk \n";
        print "parsed_Std_Sec_Disk	: $parsed_Std_Sec_Disk \n";
        print "parsed_Std_Pri_ipAddr	: $parsed_Std_Pri_ipAddr \n";
        print "parsed_Std_Sec_ipAddr	: $parsed_Std_Sec_ipAddr \n";
    }
    
    open (F, $drbdproc) || die "Could not open $drbdproc: $!";
    while ($line = <F>) {
	chomp($line);
	if ($DEBUG == "1") {
	    print $line . "\n";
	}
	next if $line =~ /^\s*$/; 		# skip blank lines
	if ($line =~ / 0: cs:(.*) st:(.*) ld:(.*)/) {
    	    $ConStatus = $1;
    	    $SyncStatus = $2;
    	    $LinkDiagnostic = $3;
    	}

        if ($line =~ /\[(.*)\] sync\'ed: (.*) \((.*)\/(.*)\)M/) {
            $sync_progress = "[" . $1 . "]";
            $sync_percentage = $2;
            $sync_current = $3;
            $sync_max = $4;
        }
        if ($line =~ /finish: (.*) speed: (.*)/) {
            $sync_finish = $1;
            $sync_speed = $2;
        }
    }
    close(F);
    if (($sync_progress == "") && ($sync_current == "") && ($sync_max == "") && ($sync_max == "")) {
	$synchronizing = "0";
    }
    else {
	$synchronizing = "1";
    }
    if ($DEBUG == "1") {
        print "\n";
        print "ConStatus		: $ConStatus \n";
        print "SyncStatus		: $SyncStatus \n";
        print "LinkDiagnostic		: $LinkDiagnostic \n";
        print "sync_progress		: $sync_progress \n";
        print "sync_percentage		: $sync_percentage \n";
        print "sync_current		: $sync_current \n";
        print "sync_max		: $sync_max \n";
        print "sync_finish		: $sync_finish \n";
        print "sync_speed		: $sync_speed \n";
        print "synchronizing		: $synchronizing \n";
    }

    open (F, $hacfgfile) || die "Could not open $hacfgfile: $!";
    while ($line = <F>) {
        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        if ($line =~ /auto_failback (.*)/) {
            $auto_failback = $1;
        }
        if ($line =~ /keepalive (.*)/) {
            $keepalive = $1;
        }
        if ($line =~ /warntime (.*)/) {
            $warntime = $1;
        }
        if ($line =~ /deadtime (.*)/) {
            $deadtime = $1;
        }
        if ($line =~ /initdead (.*)/) {
            $initdead = $1;
        }
        if ($line =~ /ping (.*)/) {
            $ping = $1;
        }
        if (($line =~ /ucast eth1 (.*)/) && (!$ucast_eth1_one)) {
            $ucast_eth1_one = $1;
        }
        if (($line =~ /ucast eth1 (.*)/) && ($ucast_eth1_one)) {
            $ucast_eth1_two = $1;
        }
        if (($line =~ /ucast eth0 (.*)/) && (!$ucast_eth0_one)) {
            $ucast_eth0_one = $1;
        }
        if (($line =~ /ucast eth0 (.*)/) && ($ucast_eth0_one)) {
            $ucast_eth0_two = $1;
        }
    }
    close(F);

    if ($DEBUG == "1") {
        print "auto_failback 		: $auto_failback \n";
        print "keepalive		: $keepalive \n";
        print "warntime        	: $warntime \n";
        print "deadtime      		: $deadtime \n";
        print "initdead      		: $initdead \n";
        print "ping    		: $ping \n";
        print "ucast_eth1_one 		: $ucast_eth1_one \n";
        print "ucast_eth1_two 		: $ucast_eth1_two \n";
        print "ucast_eth0_one 		: $ucast_eth0_one \n";
        print "ucast_eth0_two 		: $ucast_eth0_two \n";
    }
}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&validate;
	# This is where we actually populate CCE with the gathered info:

        @oids = $cce->find('Cluster');
        if ($#oids < 0) {
		if ($DEBUG == "1") {
	                print "Object not yet in CCE. Creating new one.\n";
		}
                ($ok) = $cce->create('Cluster', {
                'degr_wfc_timeout' => $degr_wfc_timeout,
                'on_disconnect' => $on_disconnect,
                'on_io_error' => $on_io_error,
                'rate' => $rate,
                'Std_Pri_fqdn' => $parsed_Std_Pri_fqdn,
                'Std_Pri_ipAddr' => $parsed_Std_Pri_ipAddr,
                'Std_Pri_Device' => $parsed_Std_Pri_Device,
                'Std_Pri_Disk' => $parsed_Std_Pri_Disk,
                'Std_Sec_fqdn' => $parsed_Std_Sec_fqdn,
                'Std_Sec_ipAddr' => $parsed_Std_Sec_ipAddr,
                'Std_Sec_Device' => $parsed_Std_Sec_Device,
                'Std_Sec_Disk' => $parsed_Std_Sec_Disk,
                'ConStatus' => $ConStatus,
                'SyncStatus' => $SyncStatus,
                'LinkDiagnostic' => $LinkDiagnostic,
                'sync_progress' => $sync_progress,
                'sync_percentage' => $sync_percentage,
                'sync_current' => $sync_current,
                'sync_max' => $sync_max,
                'sync_finish' => $sync_finish,
                'sync_speed' => $sync_speed,
                'synchronizing' => $synchronizing,

                'auto_failback' => $auto_failback,
                'keepalive' => $keepalive,
                'warntime' => $warntime,
                'deadtime' => $deadtime,
                'initdead' => $initdead,
                'ping' => $ping,
                'ucast_eth1_master' => $ucast_eth1_master,
                'ucast_eth1_slave' => $ucast_eth1_slave,
                'ucast_eth0_master' => $ucast_eth0_master,
                'ucast_eth0_slave' => $ucast_eth0_slave,

                'timestamp' => time()
                });
        }
        else {
		if ($DEBUG == "1") {
	        	print "Object already present in CCE. Updating it.\n";
		}
                # find the system object
                ($sys_oid) = $cce->find('Cluster');
                ($ok, $sys) = $cce->get($sys_oid);
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'degr_wfc_timeout' => $degr_wfc_timeout,
                'on_disconnect' => $on_disconnect,
                'on_io_error' => $on_io_error,
                'rate' => $rate,
                'Std_Pri_fqdn' => $parsed_Std_Pri_fqdn,
                'Std_Pri_ipAddr' => $parsed_Std_Pri_ipAddr,
                'Std_Pri_Device' => $parsed_Std_Pri_Device,
                'Std_Pri_Disk' => $parsed_Std_Pri_Disk,
                'Std_Sec_fqdn' => $parsed_Std_Sec_fqdn,
                'Std_Sec_ipAddr' => $parsed_Std_Sec_ipAddr,
                'Std_Sec_Device' => $parsed_Std_Sec_Device,
                'Std_Sec_Disk' => $parsed_Std_Sec_Disk,
                'ConStatus' => $ConStatus,
                'SyncStatus' => $SyncStatus,
                'LinkDiagnostic' => $LinkDiagnostic,
                'sync_progress' => $sync_progress,
                'sync_percentage' => $sync_percentage,
                'sync_current' => $sync_current,
                'sync_max' => $sync_max,
                'sync_finish' => $sync_finish,
                'sync_speed' => $sync_speed,
                'synchronizing' => $synchronizing,

                'auto_failback' => $auto_failback,
                'keepalive' => $keepalive,
                'warntime' => $warntime,
                'deadtime' => $deadtime,
                'initdead' => $initdead,
                'ping' => $ping,
                'ucast_eth1_master' => $ucast_eth1_master,
                'ucast_eth1_slave' => $ucast_eth1_slave,
                'ucast_eth0_master' => $ucast_eth0_master,
                'ucast_eth0_slave' => $ucast_eth0_slave,

                'timestamp' => time()
                });

	}

       @oids = $cce->find('Cluster_Update');
        if ($#oids < 0) {
                if ($DEBUG == "1") {
                        print "Object not yet in CCE. Creating new one.\n";
                }
                ($ok) = $cce->create('Cluster_Update', {
                'timestamp' => time()
                });
	}
        else {
                if ($DEBUG == "1") {
                        print "Object already present in CCE. Updating it.\n";
                }
                # find the system object
                ($sys_oid) = $cce->find('Cluster_Update');
                ($ok, $sys) = $cce->get($sys_oid);

                ($ok) = $cce->set($sys_oid, '',{        
                'timestamp' => time()
                });
	}
}


# Validate the settings before we pass them on to CCE:
sub validate {

    if (($degr_wfc_timeout ne "") && ($on_disconnect ne "") && ($rate ne "") && ($parsed_Std_Pri_fqdn ne "") &&
        ($parsed_Std_Sec_fqdn ne "") && ($parsed_Std_Pri_Device ne "") && ($parsed_Std_Sec_Device ne "") &&
        ($parsed_Std_Pri_Disk ne "") && ($parsed_Std_Sec_Disk ne "") && ($parsed_Std_Pri_ipAddr ne "") && 
        ($parsed_Std_Sec_ipAddr ne "")) {
	$firstblock = "ok";
    }
    if (($ConStatus ne "") && ($SyncStatus ne "") && ($LinkDiagnostic ne "")) {
	$secondblock = "ok";
	}
    if (($LinkDiagnostic ne "Consistent") && ($synchronizing == "1") && ($sync_progress ne "") && ($sync_percentage ne "") &&
        ($sync_current ne "") && ($sync_max ne "") && ($sync_finish ne "") && ($sync_speed ne "")) {
	$thirdblock = "ok";
    }
    if (($LinkDiagnostic == "Consistent") && ($synchronizing == "0")) {
	$thirdblock = "ok";
    }

    if ($ucast_eth1_one == $Std_Pri_ipAddr) {
		$ucast_eth1_master = $ucast_eth1_one;
    }
    if ($ucast_eth1_two == $Std_Sec_ipAddr) {
                $ucast_eth1_slave = $ucast_eth1_two;
    }

    @network_oid = $cce->find('Network', {'ipaddr' => $parsed_Std_Pri_ipAddr});
    if ($#network_oid ne -1) {
	($ok, $netValues) = $cce->get($network_oid[0]);
    }
    if ($netValues->{'device'} == "eth1") {
	$ucast_eth1_master = $parsed_Std_Pri_ipAddr;
    }
    @network_oid = $cce->find('Network', {'ipaddr' => $parsed_Std_Pri_ipAddr});
    if ($#network_oid ne -1) {
        ($ok, $netValues) = $cce->get($network_oid[0]);
    }
    if ($netValues->{'device'} == "eth0") {
        $ucast_eth0_master = $ucast_eth0_one;
    }

    @network_oid = $cce->find('Network', {'ipaddr' => $parsed_Std_Pri_ipAddr});
    if ($#network_oid ne -1) {
        ($ok, $netValues) = $cce->get($network_oid[0]);
	$ucast_eth0_slave = $ucast_eth0_two;
	$ucast_eth1_slave = $ucast_eth1_two;
    }

    if (($auto_failback ne "") && ($keepalive ne "") && ($warntime ne "") && ($deadtime ne "") && ($initdead ne "") &&
	($ping ne "") && ($ucast_eth1_one ne "") && ($ucast_eth1_two ne "") && ($ucast_eth0_one ne "") && 
	($ucast_eth0_two ne "") && ($ucast_eth0_master ne "") && ($ucast_eth1_master ne "") && ($ucast_eth0_slave ne "") &&
	($ucast_eth1_slave ne "")) {
        $hablock = "ok";
    }

    if ($DEBUG == "1") {

	print "ucast_eth0_master: $ucast_eth0_master \n";
	print "ucast_eth1_master: $ucast_eth1_master \n";
	print "ucast_eth0_slave: $ucast_eth0_slave \n";
	print "ucast_eth1_slave: $ucast_eth1_slave \n";

	print "1: $firstblock\n";
	print "2: $secondblock\n";
	print "3: $thirdblock\n";
	print "4: $hablock\n";
    }

    if (($firstblock ne "ok") || ($secondblock ne "ok") || ($thirdblock ne "ok") || ($hablock ne "ok")) {
	print STDERR "update_cluster_info.pl reports inconsistent data! \n";
	exit 1;
    }
}

# For DRBD 0.8:
sub main_new {

    open (F, $cfgfile) || die "Could not open $cfgfile: $!";
    while ($line = <F>) {
	chomp($line);
	next if $line =~ /^\s*$/; 		# skip blank lines
	if ($line =~ /degr-wfc-timeout (.*)\;/) {
    	    $degr_wfc_timeout = $1;
    	}
	if ($line =~ /on-disconnect (.*);/) {
    	    $on_disconnect = $1;
    	}
	else {
		$on_disconnect = "reconnect";
	}
	if ($line =~ /on-io-error (.*);/) {
    	    $on_io_error = $1;
    	}
	if ($line =~ /rate (.*)M;/) {
    	    $rate = $1 . "M";
    	}
	if ($line =~ /al-extents (.*);/) {
    	    $al_extents = $1;
    	}

	if ($line =~ /after-sb-0pri (.*);/) {
	    $after_sb_0pri = $1;
	}
	if ($line =~ /after-sb-1pri (.*);/) {
	    $after_sb_1pri = $1;
	}
	if ($line =~ /after-sb-2pri (.*);/) {
	    $after_sb_2pri = $1;
	}

	if (($line =~ /on (.*) \{/) && (!$parsed_Std_Sec_fqdn)) {
    	    $parsed_Std_Pri_fqdn = $1;
    	}
	if (($line =~ /on (.*) \{/) && ($parsed_Std_Pri_fqdn)) {
    	    $parsed_Std_Sec_fqdn = $1;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Device)) {
    	    $parsed_Std_Pri_Device = "/dev/" . $2;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Device)) {
    	    $parsed_Std_Sec_Device = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Pri_Disk = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Sec_Disk = "/dev/" . $2;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && (!$parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Pri_ipAddr = $1;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && ($parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Sec_ipAddr = $1;
    	}
    } 
    close(F);

    if ($DEBUG == "1") {
        print "degr_wfc_timeout	: $degr_wfc_timeout \n";
	print "on_disconnect		: $on_disconnect \n";
        print "rate			: $rate \n";
        print "al_extents		: $al_extents \n";

        print "after_sb_0pri		: $after_sb_0pri \n";
        print "after_sb_1pri		: $after_sb_1pri \n";
        print "after_sb_2pri		: $after_sb_2pri \n";

        print "parsed_Std_Pri_fqdn	: $parsed_Std_Pri_fqdn \n";
        print "parsed_Std_Sec_fqdn	: $parsed_Std_Sec_fqdn \n";
        print "parsed_Std_Pri_Device	: $parsed_Std_Pri_Device \n";
        print "parsed_Std_Sec_Device	: $parsed_Std_Sec_Device \n";
        print "parsed_Std_Pri_Disk	: $parsed_Std_Pri_Disk \n";
        print "parsed_Std_Sec_Disk	: $parsed_Std_Sec_Disk \n";
        print "parsed_Std_Pri_ipAddr	: $parsed_Std_Pri_ipAddr \n";
        print "parsed_Std_Sec_ipAddr	: $parsed_Std_Sec_ipAddr \n";
    }


    open (F, $drbdproc) || die "Could not open $drbdproc: $!";
    while ($line = <F>) {
        chomp($line);
        if ($DEBUG == "1") {
            print $line . "\n";
        }
        next if $line =~ /^\s*$/;               # skip blank lines

        if ($line =~ / 0: cs:(.*) st:(.*) ds:(.*)\/(.*) C r(.*)/) {
            $ConStatus = $1;
            $SyncStatus = $2;
            $LinkDiagnostic = $3;
        }

        if ($line =~ / 0: cs:(.*) ro:(.*) ds:(.*)\/(.*) C r(.*)/) {
            $ConStatus = $1;
            $SyncStatus = $2;
            $LinkDiagnostic = $3;
        }

        if ($line =~ /\[(.*)\] sync\'ed: (.*) \((.*)\/(.*)\)M/) {
            $sync_progress = "[" . $1 . "]";
            $sync_percentage = $2;
            $sync_current = $3;
            $sync_max = $4;
        }
        if ($line =~ /finish: (.*) speed: (.*)/) {
            $sync_finish = $1;
            $sync_speed = $2;
        }
    }
    close(F);
    if (($sync_progress == "") && ($sync_current == "") && ($sync_max == "") && ($sync_max == "")) {
        $synchronizing = "0";
    }
    else {
        $synchronizing = "1";
    }
    if ($DEBUG == "1") {
        print "\n";
        print "ConStatus                : $ConStatus \n";
        print "SyncStatus               : $SyncStatus \n";
        print "LinkDiagnostic           : $LinkDiagnostic \n";
        print "sync_progress            : $sync_progress \n";
        print "sync_percentage          : $sync_percentage \n";
        print "sync_current             : $sync_current \n";
        print "sync_max         	: $sync_max \n";
        print "sync_finish              : $sync_finish \n";
        print "sync_speed               : $sync_speed \n";
        print "synchronizing            : $synchronizing \n";
    }

    if (($sync_progress == "") && ($sync_current == "") && ($sync_max == "") && ($sync_max == "")) {
        $synchronizing = "0";
    }
    else {
        $synchronizing = "1";
    }
    if ($DEBUG == "1") {
        print "\n";
        print "ConStatus                : $ConStatus \n";
        print "SyncStatus               : $SyncStatus \n";
        print "LinkDiagnostic           : $LinkDiagnostic \n";
        print "sync_progress            : $sync_progress \n";
        print "sync_percentage          : $sync_percentage \n";
        print "sync_current             : $sync_current \n";
        print "sync_max         : $sync_max \n";
        print "sync_finish              : $sync_finish \n";
        print "sync_speed               : $sync_speed \n";
        print "synchronizing            : $synchronizing \n";
    }

    open (F, $hacfgfile) || die "Could not open $hacfgfile: $!";
    while ($line = <F>) {
        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        if ($line =~ /auto_failback (.*)/) {
            $auto_failback = $1;
        }
        if ($line =~ /keepalive (.*)/) {
            $keepalive = $1;
        }
        if ($line =~ /warntime (.*)/) {
            $warntime = $1;
        }
        if ($line =~ /deadtime (.*)/) {
            $deadtime = $1;
        }
        if ($line =~ /initdead (.*)/) {
            $initdead = $1;
        }
        if ($line =~ /ping (.*)/) {
            $ping = $1;
        }
        if (($line =~ /ucast eth1 (.*)/) && (!$ucast_eth1_one)) {
            $ucast_eth1_one = $1;
        }
        if (($line =~ /ucast eth1 (.*)/) && ($ucast_eth1_one)) {
            $ucast_eth1_two = $1;
        }
        if (($line =~ /ucast eth0 (.*)/) && (!$ucast_eth0_one)) {
            $ucast_eth0_one = $1;
        }
        if (($line =~ /ucast eth0 (.*)/) && ($ucast_eth0_one)) {
            $ucast_eth0_two = $1;
        }
    }
    close(F);

    if ($DEBUG == "1") {
        print "auto_failback            : $auto_failback \n";
        print "keepalive                : $keepalive \n";
        print "warntime         : $warntime \n";
        print "deadtime                 : $deadtime \n";
        print "initdead                 : $initdead \n";
        print "ping             : $ping \n";
        print "ucast_eth1_one           : $ucast_eth1_one \n";
        print "ucast_eth1_two           : $ucast_eth1_two \n";
        print "ucast_eth0_one           : $ucast_eth0_one \n";
        print "ucast_eth0_two           : $ucast_eth0_two \n";
    }

}

$cce->bye('SUCCESS');
exit(0);

